import Delta from "quill-delta-es";
import { NodeLocation } from "./location";
import type { AttributesObject, JSONNode } from "./tree";
import { CursorState } from "./cursor";
export interface InsertNodeOperation {
    op: "insert-nodes";
    location: NodeLocation;
    children: JSONNode[];
}
export interface UpdateNodeOperation {
    op: "update-attributes";
    location: NodeLocation;
    attributes: AttributesObject;
    oldAttributes: AttributesObject;
}
export interface RemoveNodeOperation {
    op: "remove-nodes";
    location: NodeLocation;
    children: JSONNode[];
}
export interface TextEditOperation {
    op: "text-edit";
    location: NodeLocation;
    id: string;
    key: string;
    delta: Delta;
    invert: Delta;
}
export declare type Operation = InsertNodeOperation | UpdateNodeOperation | RemoveNodeOperation | TextEditOperation;
export declare function invertOperation(operation: Operation): Operation;
export declare function transformOperation(a: Operation, b: Operation): Operation;
export declare function transformCursorState(base: Operation, cursorState: CursorState | null): CursorState | null;
export interface InsertNodeOperationMessage {
    op: "insert-nodes";
    location: number[];
    children: JSONNode[];
}
export interface UpdateNodeOperationMessage {
    op: "update-attributes";
    location: number[];
    attributes: AttributesObject;
    oldAttributes: AttributesObject;
}
export interface RemoveNodeOperationMessage {
    op: "remove-nodes";
    location: number[];
    children: JSONNode[];
}
export interface TextEditOperationMessage {
    op: "text-edit";
    location: number[];
    id: string;
    key: string;
    delta: Delta;
    invert: Delta;
}
export declare type OperationMessage = InsertNodeOperationMessage | UpdateNodeOperationMessage | RemoveNodeOperationMessage | TextEditOperationMessage;
export declare function operationToMessage(op: Operation): OperationMessage;
export declare function operationFromMessage(msg: OperationMessage): Operation;
