import { type BlockyNode, type BlockyDocument } from "./tree";
import { Slot } from "blocky-common/es/events";
import { type FinalizedChangeset, type ChangesetApplyOptions, type ChangesetStateLogger } from "./change";
import { NodeLocation } from "./location";
import { VersionHistory } from "./versionHistory";
import { type CursorState } from "./cursor";
export declare enum CursorStateUpdateReason {
    /**
     * The user changed the cursor manually through the changeset
     */
    changeset = "changeset",
    /**
     * handled by the browser, when the "input" event is trigger.
     */
    contentChanged = "contentChanged",
    /**
     * UI Event
     */
    uiEvent = "uiEvent"
}
export interface CursorStateUpdateEvent {
    state: CursorState | null;
    reason: CursorStateUpdateReason;
}
export declare class State implements ChangesetStateLogger {
    #private;
    readonly userId: string;
    readonly document: BlockyDocument;
    readonly beforeChangesetApply: Slot<FinalizedChangeset>;
    readonly changesetApplied: Slot<FinalizedChangeset>;
    readonly versionHistory: VersionHistory;
    readonly cursorStateChanged: Slot<CursorStateUpdateEvent>;
    constructor(userId: string, document: BlockyDocument);
    get cursorState(): CursorState | null;
    get appliedVersion(): number;
    __setCursorState(cursorState: CursorState | null, reason: CursorStateUpdateReason): void;
    getLocationOfNode(node: BlockyNode, acc?: number[]): NodeLocation;
    apply(changeset: FinalizedChangeset): boolean;
    rebase(changeset: FinalizedChangeset, options?: Partial<ChangesetApplyOptions>): FinalizedChangeset;
    findNodeByLocation(location: NodeLocation): BlockyNode;
}
