import { Slot } from "blocky-common/es/events";
import Delta from "quill-delta-es";
import type { ElementChangedEvent } from "./events";
export interface DeltaChangedEvent {
    oldDelta: Delta;
    newDelta: Delta;
}
export declare const metaKey = "#meta";
export interface AttributesObject {
    [key: string]: any;
}
export interface JSONNode {
    nodeName: string;
    id?: string;
    ["#meta"]?: AttributesObject;
    attributes?: AttributesObject;
    children?: JSONChild[];
}
export declare type JSONChild = JSONNode;
export interface BlockyNode {
    doc?: BlockyDocument;
    nodeName: string;
    parent: BlockyElement | null;
    nextSibling: BlockyNode | null;
    prevSibling: BlockyNode | null;
    firstChild: BlockyNode | null;
    lastChild: BlockyNode | null;
    childrenLength: number;
    clone(): BlockyNode;
    toJSON(): JSONNode;
}
export declare class BlockyTextModel {
    #private;
    constructor(delta?: Delta);
    get delta(): Delta;
    /**
     * Used internally.
     * If you want to modify the state of the document and
     * notify the editor to update, apply a changeset.
     */
    __applyDelta(v: Delta): void;
    toString(): string;
    get length(): number;
    clone(): BlockyTextModel;
}
interface InternAttributes {
    [key: string]: any;
}
export declare class BlockyElement implements BlockyNode {
    #private;
    nodeName: string;
    doc?: BlockyDocument;
    parent: BlockyElement | null;
    nextSibling: BlockyNode | null;
    prevSibling: BlockyNode | null;
    childrenLength: number;
    changed: Slot<ElementChangedEvent>;
    constructor(nodeName: string, attributes?: AttributesObject, children?: BlockyNode[]);
    get firstChild(): BlockyNode | null;
    get lastChild(): BlockyNode | null;
    indexOf(node: BlockyNode): number;
    queryChildByName(nodeName: string): BlockyNode | void;
    appendChild(node: BlockyNode): void;
    childAt(index: number): BlockyNode | null;
    /**
     * Used internally.
     * If you want to modify the state of the document and
     * notify the editor to update, apply a changeset.
     */
    __insertChildAt(index: number, node: BlockyNode): void;
    /**
     * Used internally.
     * If you want to modify the state of the document and
     * notify the editor to update, apply a changeset.
     */
    __setAttribute(name: string, value: string): void;
    getAttribute<T = any>(name: string): T | undefined;
    getAttributes(): InternAttributes;
    getTextModel(name: string): BlockyTextModel | undefined;
    /**
     * Used internally.
     * If you want to modify the state of the document and
     * notify the editor to update, apply a changeset.
     */
    __deleteChildrenAt(index: number, count: number): void;
    clone(): BlockyElement;
    toJSON(): JSONNode;
}
/**
 * This is a data layer of a block.
 * ID is used to locate a block in the document tree.
 *
 * A BlockElement can contain a <children-container>
 * at the end of the block to store the children.
 */
export declare class BlockElement extends BlockyElement {
    readonly id: string;
    constructor(blockName: string, id: string, attributes?: AttributesObject, children?: BlockyNode[]);
    /**
     * Used internally.
     * If you want to modify the state of the document and
     * notify the editor to update, apply a changeset.
     */
    __setAttribute(name: string, value: string): void;
    /**
     * Return the level of block,
     * not the level of [Node].
     */
    blockLevel(): number;
    clone(): BlockElement;
    cloneWithId(id: string): BlockElement;
    toJSON(): JSONNode;
}
export interface DocumentInitProps {
    title?: string;
    head?: BlockyElement;
    body?: BlockyElement;
    bodyChildren: BlockyNode[];
}
export declare class BlockyDocument extends BlockyElement {
    readonly title: BlockElement;
    readonly head: BlockyElement;
    readonly body: BlockyElement;
    readonly blockElementAdded: Slot<BlockElement>;
    readonly blockElementRemoved: Slot<BlockElement>;
    constructor(props?: Partial<DocumentInitProps>);
    reportBlockyNodeInserted(blockyNode: BlockyNode): void;
    reportBlockyNodeRemoved(blockyNode: BlockyNode): void;
}
export declare function traverseNode(node: BlockyNode, fun: (node: BlockyNode) => void): void;
export {};
